/*
 * 8 avril 2007
 * 
 * SCALP - Système de Cartographie Automatique Libre pour la Pédagogie
 * Version beta 0.94
 *
 * Copyright 1997-2007 Université de Toulouse Le Mirail / Département de Géographie / Laurent Jégou
 * Portions : 	Copyright (c) 2004 Sun Microsystems (file filter, AbstractLayout)
 * 				Copyright 1999, 2000, 2001, 2002 by Bruno Lowagie (iText pdf exports)
 * 				Copyright (C) 2001 Andrew Khan (jxl import/export format Excelé)
 * http://www.univ-tlse2.fr/geoprdc
 * jegou@univ-tlse2.fr
 * 
 * Ce programme est un logiciel libre régi par la licence CeCILL (cf. lien ci-dessous), cette licence confére
 * aux utilisateurs la liberté de modification et de redistribution du logiciel, dans le cadre d'un modéle de
 * diffusion open-source.
 * L'exercice de ces libertés est assorti de certains devoirs é la charge des utilisateurs afin de préserver
 * ce statut au cours des redistributions ultérieures.
 * Léaccessibilité au code source et les droits de copie, de modification et de redistribution qui en découlent
 * ont pour contrepartie de néoffrir aux utilisateurs quéune garantie limitée et de ne faire peser sur léauteur
 * le titulaire des droits patrimoniaux et les concédants successifs quéune responsabilité restreinte.
 * 
 * Le texte intégral de la licence se trouve dans la disribution, ou é l'adresse suivante :
 * http://www.cecill.info
 */

package carto;

import java.awt.Graphics;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.io.BufferedWriter;
import java.io.Serializable;
import java.util.Vector;

public class Polygone extends Polygon implements Cloneable, Serializable {

    public static final int X = 0;

    public static final int Y = 1;

    private Vector pointsOrigine;

    public Polygone() {
        pointsOrigine = new Vector();
    }

    /*
     * fonction de debuggage servant é afficher les coordonnées du polygone
     */
    public void afficherCoord() {
        double[] point;

        //System.out.println("Coordonnées du polygone");

        //System.out.println("taille de pointsOrigine: " +
        // pointsOrigine.size());
        //System.out.println("npoints: " + npoints);
        if (this.pointsOrigine != null) {

            for (int i = 0; i < this.pointsOrigine.size(); i++) {
                //affichage des coordonnées contenues dans le vecteur
                // pointsOrigine
                point = (double[]) this.pointsOrigine.elementAt(i);
                System.out.print(Messages.getString("Polygone.2") + point[0] + Messages.getString("Polygone.1") + point[1] + "\r\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

                //affichage des coordonnées contenues dans les tableaux xpoints
                // etypoints
                //System.out.println("X : " + this.xpoints[i] + " Y : " +
                // this.ypoints[i] +"\n");

            }

        } else {
            System.out.println(Messages.getString("Polygone.4")); //$NON-NLS-1$
        }
    }

    /*
     * fonction qui permet d'ajouter un point é un polygone
     */
    public void ajouterPoint(double[] p) {
        this.pointsOrigine.add(p);
    }

    /*
     * méthode permettant de redimensionner un polygone en fonction du
     * coefficient coeff
     */
    public void calculerCoord(int larg, int haut, double coeff) {
        double[] point;

        //System.out.println("taille po: " + this.pointsOrigine.size());
        this.npoints = this.pointsOrigine.size();
        //System.out.println("taille np: " + this.npoints);

        this.xpoints = new int[this.npoints];
        this.ypoints = new int[this.npoints];

        for (int i = 0; i < this.npoints; i++) {

            point = (double[]) this.pointsOrigine.elementAt(i);

            //System.out.println("coeff: "+ coeff); //debug

            this.xpoints[i] = (int) (point[X] * coeff) + larg;
            this.ypoints[i] = (int) -(point[Y] * coeff) + haut;

            // methode heritée qui ajoute un point en mettant é
            // jour les champs xpoints ypoints npoints
            //System.out.print("coordonnées:" + pX + " " + pY + "\n\n");
            // //debug
        }
    }

    /*
     * redéfinition de la méthode clone()
     */
    public Object clone() {

        try {
            Polygone poly = (Polygone) super.clone();
            if (this.npoints != 0) {
                for (int i = 0; i < this.pointsOrigine.size(); i++) {
                    poly.pointsOrigine.add(i, ((Point2D) ((Point2D) this.pointsOrigine.get(i)).clone()));
                    //System.out.println("compteur poly : " + i + "\n");
                }
            }
            return poly;
        } catch (CloneNotSupportedException x) {
            return (new InternalError(Messages.getString("Polygone.0"))); //$NON-NLS-1$
        }
    }

    /*
     * fonction qui adapte les coordonnées recupérées d'un format MIF au systeme
     * de coordonnées des fenétres Java
     */
    public void decalerCoord(double Xmin, double Ymax) {
        double[] point;

        for (int i = 0; i < this.pointsOrigine.size(); i++) {
            point = (double[]) this.pointsOrigine.get(i);

            point[X] = point[X] - Xmin;
            point[Y] = point[Y] - Ymax;

            this.pointsOrigine.setElementAt(point, i);

        }
    }

    /* Méthode de dessin d'un polygone */
    public void drawPolygone(Graphics g) {
        g.drawPolygon(this);
    }

    /*
     * Enregistrement d'un polygone au format texte
     */
    public void enregistrerPoly(BufferedWriter tampon) {

        try {
            String s = new String();

            for (int i = 0; i < this.npoints; i++) {
                double[] point = (double[]) this.pointsOrigine.get(i);
                tampon.write(String.valueOf(point[X]));
                tampon.write(";"); //$NON-NLS-1$
                tampon.write(String.valueOf(point[Y]));
                tampon.newLine();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /*
	 * Calcul des coordonnées du point central du polygone
	 */
	public double[] getCentroide() {
		int i;
		int nb = this.pointsOrigine.size();
		double sx = 0;
		double sy = 0;
		double[] pec;
		double[] centre = {0, 0};
		for (i = 0; i < nb; i++) {
			pec = (double[])this.pointsOrigine.elementAt(i); 
			sx += pec[0];
			sy += pec[1];
		}
		centre[0] = sx / nb;
		centre[1] = sy / nb;
		return centre;
	}
	
	/*
     * fonction qui va renvoyer un Vector de 4 éléments contenant les
     * coordonnées extrémes de la carte le vector renvoyé contiendra : en
     * position 0, la coordonnée la plus é gauche (x) en position 1, la
     * coordonnée la plus en haut (y) en position 2, la coordonnée la plus é
     * gauche (x) en position 3, la coordonnée la plus en bas (y)
     */
    public double[] trouverExtremes() {
        double[] extremes = new double[4];
        double[] point = (double[]) this.pointsOrigine.elementAt(0);

        double x;
        double y;

        x = point[X];
        y = point[Y];

        //initialisation du tableau extremes[]
        extremes[0] = x;
        extremes[1] = y;
        extremes[2] = x;
        extremes[3] = y;

        for (int i = 1; i < this.pointsOrigine.size(); i++) {

            point = (double[]) this.pointsOrigine.get(i);

            x = point[X];
            y = point[Y];

            if (x < extremes[0])
                extremes[0] = x;
            if (x > extremes[2])
                extremes[2] = x;

            if (y > extremes[1])
                extremes[1] = y;
            if (y < extremes[3])
                extremes[3] = y;
        }

        return extremes;
    }

    public double[] trouverLimites() {
    	double[] extremes = new double[4];
    	Rectangle r = this.getBounds();
    	extremes[0] = r.getMinX();
    	extremes[1] = r.getMaxY();
    	extremes[2] = r.getMaxX();
    	extremes[3] = r.getMinY();
    	return extremes;
    }
    
    public int getNbPts() {
    	return this.pointsOrigine.size();
    }
    
}