/*
 * 8 avril 2007
 * 
 * SCALP - Système de Cartographie Automatique Libre pour la Pédagogie
 * Version beta 0.94
 *
 * Copyright 1997-2007 Université de Toulouse Le Mirail / Département de Géographie / Laurent Jégou
 * Portions : 	Copyright (c) 2004 Sun Microsystems (file filter, AbstractLayout)
 * 				Copyright 1999, 2000, 2001, 2002 by Bruno Lowagie (iText pdf exports)
 * 				Copyright (C) 2001 Andrew Khan (jxl import/export format Excelé)
 * http://www.univ-tlse2.fr/geoprdc
 * jegou@univ-tlse2.fr
 * 
 * Ce programme est un logiciel libre régi par la licence CeCILL (cf. lien ci-dessous), cette licence confére
 * aux utilisateurs la liberté de modification et de redistribution du logiciel, dans le cadre d'un modéle de
 * diffusion open-source.
 * L'exercice de ces libertés est assorti de certains devoirs é la charge des utilisateurs afin de préserver
 * ce statut au cours des redistributions ultérieures.
 * Léaccessibilité au code source et les droits de copie, de modification et de redistribution qui en découlent
 * ont pour contrepartie de néoffrir aux utilisateurs quéune garantie limitée et de ne faire peser sur léauteur
 * le titulaire des droits patrimoniaux et les concédants successifs quéune responsabilité restreinte.
 * 
 * Le texte intégral de la licence se trouve dans la disribution, ou é l'adresse suivante :
 * http://www.cecill.info
 */

package donnees;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.EOFException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import org.nfunk.jep.JEP;

public class TableModel extends AbstractTableModel {
	protected Vector data = new Vector(); //données

	protected Object[][] tableau;

	protected Vector columnNames; //champDonnees

	protected String datafile; //nom du fichier de données

	/** Creates a new instance of DataFileTableModel */
	public TableModel(String action, String dataFile) {
		if (action == "importerMID") { //$NON-NLS-1$
			importerMID(dataFile);
		}
		if (action == "ouvrir") { //$NON-NLS-1$
			ouvrir(dataFile);
		}
	}

	private void ouvrir(String dataFile) {
		int i = 0;
		int j = 0;
		int nbChamps = 0;
		int nbDonnees = 0;
		int nbEnrs = 0;
		int posD = 0;

		try {
			BufferedReader fileInput = new BufferedReader(new InputStreamReader(new FileInputStream(dataFile)));
			String line = ""; //$NON-NLS-1$
			line = fileInput.readLine();

			if (line.startsWith("é") == false) { //Decalage //$NON-NLS-1$
				line = fileInput.readLine();
			}
			nbChamps = Integer.parseInt(line.substring(1, line.indexOf("Champs") - 1)); //$NON-NLS-1$
			columnNames = new Vector(nbChamps);

			for (i = 0; i < nbChamps; i++) { //Lecture champs
				champDonnees cdt = new champDonnees();
				line = fileInput.readLine();
				posD = line.indexOf(";"); //Délimiteur //$NON-NLS-1$
				cdt.nom = line.substring(0, posD);
				cdt.typeI = Integer.parseInt(line.substring(posD + 1));
				cdt.typeS = cdt.getTypeFromInt(cdt.typeI);
				columnNames.add(cdt);
			}
			line = fileInput.readLine(); //En-téte données
			nbDonnees = Integer.parseInt(line.substring(1, line.indexOf("Donnees") - 1)); //$NON-NLS-1$
			data = new Vector(nbDonnees);
			nbEnrs = nbDonnees / nbChamps;
			tableau = new Object[nbEnrs][nbChamps];
			int ligne = 0;
			for (i = 0; i < nbEnrs; i++) {
				line = fileInput.readLine();
				//String enregistrement[] = new String[nbChamps];
				StringTokenizer dataTokens = new StringTokenizer(line, ";"); //$NON-NLS-1$
				j = 0;
				while (dataTokens.hasMoreTokens()) {
					Object nt = dataTokens.nextToken();
					if (((String) nt).equals("null")) {
						tableau[ligne][j] = null;
						data.add(null);
					} else {
						tableau[ligne][j] = nt;                   
						//enregistrement[j] = (String)tableau[ligne][j];
						switch (((champDonnees) columnNames.get(j)).typeI) {
						case 99: //type non reconnu : String
							data.add((String) tableau[ligne][j]);
							break;
						case 0: //Integer
							data.add(new Integer(Integer.parseInt((String) tableau[ligne][j])));
							break;
						case 1: //Char
							data.add((String) tableau[ligne][j]);
							break;
						case 2: //Float
							data.add(new Double(Double.parseDouble((String) tableau[ligne][j])));
							break;
						}
					}
					j++;
				}
				ligne++;
			}

		} catch (NoSuchElementException e) {
			System.out.println(Messages.getString("TableModel.8") + e.getMessage()); //$NON-NLS-1$
			System.exit(1);
		} catch (IOException e) {
			System.out.println(Messages.getString("TableModel.9") + e.getMessage()); //$NON-NLS-1$
			System.exit(1);
		}
	}

	private void importerMID(String dataFile) {
		try {
			String fichierMif = dataFile;
			String fichierMid = dataFile.substring(0, dataFile.length() - 4) + ".MID"; //$NON-NLS-1$
			String ligneMif;
			String ligneMid;
			String delimiteur;

			int positionCar;
			int positionCar2;
			int nbColonnes = 0;
			int nbLignes = 0;

			Vector donnees = new Vector(); // contient les données du fond

			Vector champ = new Vector(); // Vector qui corespond é un champ
			Vector typeChamp = new Vector(); // Enregistrement type de champ

			FileReader entreeMif = new FileReader(fichierMif);
			BufferedReader tamponMif = new BufferedReader(entreeMif);

			//Verification que le fichier MIF n'est pas vide
			String debut = tamponMif.readLine().subSequence(0, 7).toString().toUpperCase();
			if (!debut.equals("VERSION")) //$NON-NLS-1$
				System.out.println(Messages.getString("TableModel.12")); //$NON-NLS-1$
			else {

				//recherche de la ligne contenant le delimiteur
				ligneMif = tamponMif.readLine();
				String delim = ligneMif.subSequence(0, 9).toString().toUpperCase();
				while (!delim.equals("DELIMITER")) { //$NON-NLS-1$
					ligneMif = tamponMif.readLine();
					delim = ligneMif.subSequence(0, 9).toString().toUpperCase(); //Décalage
				}

				positionCar = ligneMif.indexOf(" "); //$NON-NLS-1$
				delimiteur = ligneMif.substring(positionCar + 2, positionCar + 3);
				//System.out.println(delimiteur); //debug

				//tant que l'on a pas atteint la ligne Columns on avance
				delim = ligneMif.subSequence(0, 7).toString().toUpperCase();
				while (!delim.equals("COLUMNS")) { //$NON-NLS-1$
					ligneMif = tamponMif.readLine();
					delim = ligneMif.subSequence(0, 7).toString().toUpperCase();
				}

				//ligneMif = tamponMif.readLine();

				if (ligneMif.equals("data")  || ligneMif.equals("DATA")) //$NON-NLS-1$
					System.out.println(Messages.getString("TableModel.17")); //$NON-NLS-1$
				else {
					ligneMif = tamponMif.readLine();
					if (ligneMif.equals("data")  || ligneMif.equals("DATA")) //$NON-NLS-1$
						System.out.println(Messages.getString("TableModel.19")); //$NON-NLS-1$
					else {
						// tant que l'on a pas atteint la ligne Data on
						// avance
						// et on met dans la premiére colonne des Vecteurs
						// correspondant aux
						// champs de données le nom de la colonne de données
						//ligneMif = tamponMif.readLine();
						while (!ligneMif.toUpperCase().equals("DATA") && ligneMif.length() != 0) { //$NON-NLS-1$
							nbColonnes++;
							//while (ligneMif.startsWith(" ")) { //$NON-NLS-1$
							//	ligneMif = ligneMif.substring(1);
							//}
							ligneMif = ligneMif.trim();
							positionCar = ligneMif.indexOf(" "); //$NON-NLS-1$
							champ.add(ligneMif.substring(0, positionCar));
							typeChamp.add(ligneMif.substring(positionCar + 1));
							ligneMif = tamponMif.readLine();
							//donnees.add(champ);
						}

						// System.out.println(ligne); //debug

						tamponMif.close();

						FileReader entreeMid = new FileReader(fichierMid);
						BufferedReader tamponMid = new BufferedReader(entreeMid);
						ligneMid = tamponMid.readLine();// permiére ligne du
						// fichier

						// Si le fichier ne contient pas le délimiteur ce n'est
						// pas un fichier MID
						// ou alors ce fichier ne contient qu'un seul champ de
						// données donc il ne
						// peut pas étre importé
						if (ligneMid.indexOf(delimiteur) == -1) {
							System.out.println(Messages.getString("TableModel.22")); //$NON-NLS-1$
						} else {
							while (ligneMid != null && !(ligneMid.equals(""))) { //$NON-NLS-1$
								//positionCar = ligneMid.indexOf(delimiteur);
								String enregistrement[] = new String[nbColonnes];
								if (nbColonnes > 0) {
									//positionCar2 =
									// ligneMid.indexOf(delimiteur,
									// positionCar + 1);//2e
									int d1 = 0;
									int d2 = 0;
									positionCar = 0;
									String dq = ""; //$NON-NLS-1$
									for (int i = 0; i < nbColonnes - 1; i++) {
										d1 = 0;
										d2 = 0;
										if (ligneMid.charAt(positionCar) == delimiteur.toCharArray()[0]) {
											positionCar++;
										}
										if (ligneMid.charAt(positionCar) == '"' && positionCar != ligneMid.length()) {
											d1++;
										}
										positionCar2 = ligneMid.indexOf(delimiteur, positionCar + 1);//prochaine
										if (ligneMid.charAt(positionCar2 - 1) == '"' && positionCar2 != 0) {
											d2--;
										}
										//System.out.println("p1 : " +
										// ligneMid.charAt(positionCar));
										//System.out.println("p2 : " +
										// ligneMid.charAt(positionCar2));
										dq = ligneMid.substring(positionCar + d1, positionCar2 + d2);
										if (dq.length() == 0) {
											dq = null;
										}
										if (dq != null) {
											enregistrement[i] = dq;
										} else {
											enregistrement[i] = ""; //$NON-NLS-1$
										}
										//System.out.println(enregistrement[i]);
										positionCar = positionCar2;
									}
									dq = (ligneMid.substring(positionCar + 1));
									if (dq.length() == 0 || dq.equals("\"\"")) {
										dq = "";
									}
									enregistrement[nbColonnes - 1] = dq;
									donnees.add(enregistrement.clone());
									nbLignes++;
								}
								ligneMid = tamponMid.readLine();
							}
						}

						//nbLignes--;
						tamponMid.close();
						int i = 0;
						int j = 0;
						columnNames = new Vector(nbColonnes);
						champDonnees cdt;
						for (i = 0; i < nbColonnes; i++) {
							cdt = new champDonnees();
							cdt.nom = champ.get(i).toString();
							cdt.typeS = typeChamp.get(i).toString();
							cdt.typeI = getTypeMID(cdt.typeS);
							columnNames.add(cdt);
						}

						tableau = new Object[nbLignes][nbColonnes];
						for (i = 0; i < nbLignes; i++) {
							Object enrr[] = (Object[]) donnees.get(i);
							for (j = 0; j < nbColonnes; j++) {
								//tableau[i][j] = enrr[j];
								switch (getTypeMID((String) typeChamp.get(j))) {
								case 99: //type non reconnu : String
								data.add(enrr[j]);
								break;
								case 0: //Integer
								data.add(new Integer(Integer.parseInt((String) enrr[j])));
								break;
								case 1: //Char
									data.add((String) enrr[j]);
									break;
								case 2: //Float
									data.add(new Double(Double.parseDouble((String) enrr[j])));
									break;
								}
							}
						}

					}

				}

			}

		} catch (FileNotFoundException e) {
			// Exception déclenchée si le fichier n'existe pas
			System.out.println(Messages.getString("TableModel.24")); //$NON-NLS-1$
		} catch (EOFException e) {
			// Exception déclenchée si la fin du fichier est atteinte
			System.out.println(Messages.getString("TableModel.25")); //$NON-NLS-1$
		} catch (IOException e) {
			// Exception déclenchée si un autre probléme survient
			// pendant l'accés au fichier
			System.out.println(Messages.getString("TableModel.26")); //$NON-NLS-1$
		}
	}

	public TableModel(String action, int champ, Vector data, Vector columnNames) {
		if (action == "suppr.champ") { //$NON-NLS-1$
			this.columnNames = columnNames;
			this.data = data;
			int i = 0, j = 0;
			int nbCols = columnNames.size();
			int nbEnrs = data.size() / columnNames.size();
			this.columnNames.remove(champ);
			//Mise é jour des données
			Vector dataTemp = (Vector) data.clone();
			data = new Vector((nbCols - 1) * (data.size() / (nbCols - 1)));
			for (i = 0; i < nbEnrs; i++) {
				for (j = 0; j < nbCols; j++) {
					if (j != champ)
						data.add(dataTemp.get(i * nbCols + j));
				}
			}
			this.data = data;
			dataTemp = null;
		}
	}

	public TableModel(String action, String champ, Vector data, Vector columnNames) {

		if (action.equals("creeChamp")) { //$NON-NLS-1$
			try {
				int i = 0, j = 0;
				int nbCols = columnNames.size();
				int nbEnrs = data.size() / columnNames.size();

				this.columnNames = columnNames;
				Vector dataTemp = (Vector) data.clone();
				data = new Vector((nbCols + 1) * nbEnrs);

				// Modification de l'attribut columnNames
				champDonnees nChamp = new champDonnees();
				// dataFile contient le type(int) et le nom du nouveau champ
				nChamp.typeI = Integer.parseInt(champ.substring(0, 1));
				nChamp.typeS = nChamp.getTypeFromInt(((champDonnees) columnNames.get(nbCols - 1)).typeI);
				nChamp.nom = champ.substring(1);
				this.columnNames.addElement(nChamp);

				// Modification de l'attribut tableau
				for (i = 0; i < nbEnrs; i++) {
					for (j = 0; j < nbCols; j++) {
						data.add(dataTemp.get(i * nbCols + j));
					}
					data.add(new Double(0));
				}
				this.data = data;
			} catch (NoSuchElementException e) {
				System.out.println(Messages.getString("TableModel.29") + e.getMessage()); //$NON-NLS-1$
			}
		}
	}

//	public TableModel(String action, String champACreer, Vector data, Vector columnNames, String formule) {
//	if (action.equals("projection")) { //$NON-NLS-1$
//	try {

//	} catch (NoSuchElementException e) {
//	System.out.println(Messages.getString("TableModel.31") + e.getMessage()); //$NON-NLS-1$
//	System.exit(1);
//	}
//	}
//	}

	public TableModel(String action, String champCR, String champACreer, Vector data, Vector columnNames) {
		if (action.equals("centrage_reduction")) { //$NON-NLS-1$
			try {
				this.columnNames = columnNames;
				this.data = data;
				int i = 0, j = 0;
				int nbCols = columnNames.size() + 1;
				int nbEnrs = data.size() / columnNames.size();
				int numChampCR = getNumChamp(champCR);

				//Calcul moyenne et ec du champ
				double moyenne = getMoyenne(numChampCR);
				double ec = getEC(numChampCR, moyenne);
				double[] valeur = new double[nbEnrs];
				for (i = 0; i < nbEnrs; i++) {
					switch (((champDonnees) columnNames.get(numChampCR)).typeI) {
					case 0:
						valeur[i] = (((Integer) getValueAt(i, numChampCR)).doubleValue() - moyenne) / ec;
						break;
					case 1:
						valeur[i] = (((Double) getValueAt(i, numChampCR)).doubleValue() - moyenne) / ec;
						break;
					}
				}

				// Modification de l'attribut columnNames
				champACreer = "2" + champACreer; //Ajout type double devant //$NON-NLS-1$
				// //$NON-NLS-1$
				champDonnees nouveau = new champDonnees();
				nouveau.typeI = Integer.parseInt(champACreer.substring(0, 1));
				nouveau.typeS = nouveau.getTypeFromInt(nouveau.typeI);
				nouveau.nom = champACreer.substring(1);
				columnNames.add(nouveau);
				this.columnNames = columnNames;

				//Mise é jour des données
				Vector dataTemp = (Vector) data.clone();
				data = new Vector(nbCols * nbEnrs);
				for (i = 0; i < nbEnrs; i++) {
					for (j = 0; j < (nbCols - 1); j++) {
						data.add(dataTemp.get(i * (nbCols - 1) + j));
					}
					data.add(new Double(valeur[i]));
				}
				this.data = data;
				dataTemp = null;
			} catch (NoSuchElementException e) {
				System.out.println(Messages.getString("TableModel.34") + e.getMessage()); //$NON-NLS-1$
				System.exit(1);
			}
		}
	}

	public TableModel(String action, String champACreer, Vector data, Vector columnNames, String formule) {
		if (action.equals("calulerChamp")) { //$NON-NLS-1$
			try {
				this.columnNames = columnNames;
				this.data = data;
				int i = 0, j = 0;
				int nbCols = columnNames.size() + 1;
				int nbEnrs = data.size() / columnNames.size();

				double[] valeur = new double[nbEnrs];
				for (i = 0; i < nbEnrs; i++) {
					valeur[i] = evalueFormule(i, formule);
				}

				
			// Modification de l'attribut columnNames
			champACreer = "2" + champACreer; //Ajout type double devant //$NON-NLS-1$
			// //$NON-NLS-1$
			champDonnees nouveau = new champDonnees();
			nouveau.typeI = Integer.parseInt(champACreer.substring(0, 1));
			nouveau.typeS = nouveau.getTypeFromInt(nouveau.typeI);
			nouveau.nom = champACreer.substring(1);
			columnNames.add(nouveau);
			this.columnNames = columnNames;

			//Mise é jour des données
			Vector dataTemp = (Vector) data.clone();
			data = new Vector(nbCols * nbEnrs);
			for (i = 0; i < nbEnrs; i++) {
				for (j = 0; j < (nbCols - 1); j++) {
					data.add(dataTemp.get(i * (nbCols - 1) + j));
				}
				data.add(new Double(valeur[i]));
			}
			this.data = data;
			dataTemp = null;
		} catch (NoSuchElementException e) {
			System.out.println(Messages.getString("TableModel.36") + e.getMessage()); //$NON-NLS-1$
			System.exit(1);
		}
	}
}

private Double evalueFormule(Integer r, String formule) {
	Integer i = 0;
	JEP myParser = new JEP();
	String formuleBase = formule;
	String formuleVa = formuleBase;
	for (i = 0; i < columnNames.size(); i++) {
		String nomChamp = ((champDonnees) columnNames.get(i)).nom;
		if (formuleVa.lastIndexOf(nomChamp) != -1) {
			String valeur = getValueAt(r, i).toString();
			String tf = formuleVa.replaceAll(nomChamp, valeur);
			formuleVa = tf;
		}
	}
	myParser.parseExpression(formuleVa);
	return myParser.getValue();
}

/*
 * Fonction provisoire en attendant d'inclure une ligne définissant les
 * types des variables dans le fichier source
 */
String typeDesVariables(StringTokenizer var, int numcol) {
	String rep = ""; //$NON-NLS-1$
	int i = 0;
	while (var.hasMoreElements() == true) {
		if (i == numcol) {
			if (var.nextToken().compareTo("int") == 0) //$NON-NLS-1$
				rep = "int"; //$NON-NLS-1$
			if (var.nextToken().compareTo("str") == 0) //$NON-NLS-1$
				rep = "str"; //$NON-NLS-1$
		} else
			i++;
	}
	return rep;
}

public void enregistrerDonnees(String fichier) {
	try {

		FileWriter fw = new FileWriter(fichier);
		BufferedWriter bw = new BufferedWriter(fw);

		int i = 0;
		int j = 0;
		String ligne = ""; //$NON-NLS-1$

		bw.write("//Fichier de données SCAPJ"); //$NON-NLS-1$
		bw.newLine();
		bw.write("é" + columnNames.size() + " Champs"); //$NON-NLS-1$ //$NON-NLS-2$
		bw.newLine();
		for (j = 0; j < columnNames.size(); j++) {
			ligne = ((champDonnees) columnNames.get(j)).nom;
			ligne = ligne + ";"; //$NON-NLS-1$
			ligne = ligne + ((champDonnees) columnNames.get(j)).typeI;
			bw.write(ligne);
			bw.newLine();
		}

		bw.write("é" + data.size() + " Donnees"); //$NON-NLS-1$ //$NON-NLS-2$
		bw.newLine();
		for (i = 0; i < data.size(); i++) {
			if (data.elementAt(i) != null) {
				ligne = data.elementAt(i).toString();
				if (ligne.equals("\"\"")  || ligne.length() == 0) ligne = "null";
				ligne = ligne + ";"; //$NON-NLS-1$
				bw.write(ligne);
			} else
				bw.write("null;"); //$NON-NLS-1$
			if ((i + 1) % j == 0) {
				bw.newLine();
			}
		}
		bw.close();
		fw.close();
	} catch (NoSuchElementException e) {
		System.out.println(Messages.getString("TableModel.51") + e.getMessage()); //$NON-NLS-1$
		System.exit(1);
	} catch (IOException e) {
		System.out.println(Messages.getString("TableModel.52") + e.getMessage()); //$NON-NLS-1$
		System.exit(1);
	}
}

public TableModel(String action, Vector columnNames, Vector data, Vector newC, Vector newT) {
	if (action.equals("importXLS")) { //$NON-NLS-1$
		try {
			int i = 0, j = 0, k = 0;
			int nbCols = columnNames.size();
			int nbEnrs = data.size() / columnNames.size();

			// Modification de l'attribut columnNames
			columnNames.addAll(newC);
			this.columnNames = columnNames;
			this.data = data;
			Vector dataTemp = (Vector) data.clone();
			data = new Vector((newC.size()) * nbEnrs);
			int lignes = newT.size() / (newC.size() + 1);
			boolean trouve; //Si nouvelles données a ajouter trouvées.
			// Modification de l'attribut tableau
			for (i = 0; i < nbEnrs; i++) {
				trouve = false;
				for (j = 0; j < nbCols; j++) {
					data.add(dataTemp.get((i * nbCols) + j));
				}
				for (j = 0; j < lignes; j++) {
					if (((String) dataTemp.elementAt(i * nbCols)).equals(newT.elementAt(j * (newC.size() + 1)))) {
						trouve = true;
						for (k = 1; k <= newC.size(); k++) {
							data.add(newT.elementAt((j * (newC.size() + 1)) + k));
						}
						break;
					}
				}
				if (trouve == false) {
					for (k = 0; k < newC.size(); k++) {
						switch (((champDonnees) newC.elementAt(k)).typeI) {
						case 0:
							data.add(new Integer(0));
							break;
						case 1:
							data.add(""); //$NON-NLS-1$
							break;
						case 2:
							data.add(new Double(0));
							break;
						}
					}
				}
			}
			this.data = data;
		} catch (NoSuchElementException e) {
			System.out.println(Messages.getString("TableModel.54") + e.getMessage()); //$NON-NLS-1$
		}
	}
} /* Méthodes d'accés é l'attribut columnNames */

public Vector getColumnNames() {
	return (columnNames);
}

/* Méthodes d'accés é l'attribut tableau */
public Object[][] getTableau() {
	return (tableau);
}

public Vector getData() {
	return (data);
}

/*
 * Méthode de conversion pour passer d'un attribut String é double. Cette
 * méthode est utilisée les sous classes de ModeleTable
 */
public static double strToDouble(String s) {
	Double res = new Double(s);
	double d = res.doubleValue();
	return d;
}

public int getColumnCount() {
	return columnNames.size();
}

public int getRowCount() {
	return data.size() / getColumnCount();
}

public String getColumnName(int columnIndex) {
	String colName = ""; //$NON-NLS-1$
	if (columnIndex <= getColumnCount())
		colName = ((champDonnees) (columnNames.get(columnIndex))).nom;
	return colName;
}

public Class getColumnClass(int c) {
	return getValueAt(0, c).getClass();
}

public boolean isCellEditable(int rowIndex, int columnIndex) {
	if (columnIndex > 1) {
		return true;
	} else {
		return false;
	}
}

public Object getValueAt(int rowIndex, int columnIndex) {
	Object cellObjet = data.elementAt((rowIndex * getColumnCount()) + columnIndex);
	if (cellObjet.getClass().toString().equals("class java.lang.Double")) {
		Long valeur =  Math.round((Double) cellObjet * 100);
		cellObjet = valeur.doubleValue() / 100;
	}
	return cellObjet;
}

public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
	//if (aValue != null) {
	data.setElementAt(aValue, (rowIndex * getColumnCount()) + columnIndex);
	//} else {
	//    Integer a = new Integer(0);
	//    data.setElementAt(a, (rowIndex * getColumnCount()) + columnIndex);
	//}
}

public int getTypeMID(String nomChamp) {
	nomChamp = nomChamp.trim();
	nomChamp = nomChamp.toUpperCase();
	if (nomChamp.equals("INTEGER")) { //$NON-NLS-1$
		return 0;
	}
	if (nomChamp.substring(0, 4).equals("CHAR")) { //$NON-NLS-1$
		return 1;
	}
	if (nomChamp.equals("FLOAT")) { //$NON-NLS-1$
		return 2;
	}
	if (nomChamp.length() >= 7 && nomChamp.substring(0, 7).equals("DECIMAL")) { //$NON-NLS-1$
		return 2;
	}
	return 99;
}

public String[] getNomsChamps() {
	int i = 0;
	String[] noms = new String[columnNames.size()];
	for (i = 0; i < columnNames.size(); i++) {
		noms[i] = ((champDonnees) columnNames.get(i)).nom;
	}
	return noms;
}

private int getNumChamp(String champ) {
	int i = 0;
	for (i = 0; i < columnNames.size(); i++) {
		if (((champDonnees) columnNames.get(i)).nom.equals(champ))
			return i;
	}
	return 99;
}

public double getMoyenne(int champ) {
	int i = 0;
	int nbEnrs = data.size() / columnNames.size();
	double somme = 0;
	for (i = 0; i < nbEnrs; i++) {
		switch (((champDonnees) columnNames.get(champ)).typeI) {
		case 0:
			somme += ((Integer) getValueAt(i, champ)).doubleValue();
			break;
		case 2:
			somme += ((Double) getValueAt(i, champ)).doubleValue();
			break;
		}
	}
	return (somme / nbEnrs);
}

public double getEC(int champ, double moyenne) {
	int i = 0;
	int nbEnrs = data.size() / columnNames.size();
	double sommeCEM = 0;
	for (i = 0; i < nbEnrs; i++) {
		switch (((champDonnees) columnNames.get(champ)).typeI) {
		case 0:
			sommeCEM += Math.pow(moyenne - ((Integer) getValueAt(i, champ)).doubleValue(), 2);
			break;
		case 2:
			sommeCEM += Math.pow(moyenne - ((Double) getValueAt(i, champ)).doubleValue(), 2);
			break;
		}
	}
	return (Math.sqrt((sommeCEM / nbEnrs)));
}

}