/*
 * 8 avril 2007
 * 
 * SCALP - Système de Cartographie Automatique Libre pour la Pédagogie
 * Version beta 0.94
 *
 * Copyright 1997-2007 Université de Toulouse Le Mirail / Département de Géographie / Laurent Jégou
 * Portions : 	Copyright (c) 2004 Sun Microsystems (file filter, AbstractLayout)
 * 				Copyright 1999, 2000, 2001, 2002 by Bruno Lowagie (iText pdf exports)
 * 				Copyright (C) 2001 Andrew Khan (jxl import/export format Excelé)
 * http://www.univ-tlse2.fr/geoprdc
 * jegou@univ-tlse2.fr
 * 
 * Ce programme est un logiciel libre régi par la licence CeCILL (cf. lien ci-dessous), cette licence confére
 * aux utilisateurs la liberté de modification et de redistribution du logiciel, dans le cadre d'un modéle de
 * diffusion open-source.
 * L'exercice de ces libertés est assorti de certains devoirs é la charge des utilisateurs afin de préserver
 * ce statut au cours des redistributions ultérieures.
 * Léaccessibilité au code source et les droits de copie, de modification et de redistribution qui en découlent
 * ont pour contrepartie de néoffrir aux utilisateurs quéune garantie limitée et de ne faire peser sur léauteur
 * le titulaire des droits patrimoniaux et les concédants successifs quéune responsabilité restreinte.
 * 
 * Le texte intégral de la licence se trouve dans la disribution, ou é l'adresse suivante :
 * http://www.cecill.info
 */

package guiInterface;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.PathIterator;
import java.awt.geom.Rectangle2D;

/**
 * @author Laurent
 *  
 */
public class GraphicObject {

    protected Point m_position = new Point(0, 0);

    protected Dimension m_size = new Dimension(0, 0);

    protected double m_rotation = 0;

    protected boolean m_autoSize = false;

    protected GraphicObjectsCollection m_container;

    protected String m_type;

    //Constructeurs

    public GraphicObject() {
        //Constructeur vide
    }

    public GraphicObject(Point position) {
        this.setPosition(position);
    }

    public GraphicObject(int posX, int posY) {
        this.setX(posX);
        this.setY(posY);
    }

    public GraphicObject(Point position, Dimension size) {
        this.setPosition(position);
        this.setSize(size);
    }

    public GraphicObject(int posX, int posY, Dimension size) {
        this.setX(posX);
        this.setY(posY);
        this.setSize(size);
    }

    public GraphicObject(int posX, int posY, int width, int height) {
        this.setX(posX);
        this.setY(posY);
        this.setWidth(width);
        this.setHeight(height);
    }

    public GraphicObject(Point position, double rotation) {
        this.setPosition(position);
        this.setRotation(rotation);
    }

    public GraphicObject(int posX, int posY, double rotation) {
        this.setX(posX);
        this.setY(posY);
        this.setRotation(rotation);
    }

    public GraphicObject(Point position, Dimension size, double rotation) {
        this.setPosition(position);
        this.setSize(size);
        this.setRotation(rotation);
    }

    public GraphicObject(int posX, int posY, int width, int height, double rotation) {
        this.setX(posX);
        this.setY(posY);
        this.setWidth(width);
        this.setHeight(height);
        this.setRotation(rotation);
    }

    //Propriétés

    public GraphicObjectsCollection getContainer() {
        return m_container;
    }

    public void setContainer(GraphicObjectsCollection valeur) {
        m_container = valeur;
    }

    public int getX() {
        return m_position.x;
    }

    public void setX(int valeur) {
        m_position.x = valeur;
    }

    public int getY() {
        return m_position.y;
    }

    public void setY(int valeur) {
        m_position.y = valeur;
    }

    public boolean getAutoSize() {
        return m_autoSize;
    }

    public void setAutoSize(boolean valeur) {
        m_autoSize = valeur;
    }

    public Point getPosition() {
        return m_position;
    }

    public void setPosition(Point valeur) {
        m_position = valeur;
    }

    public int getWidth() {
        return m_size.width;
    }

    public void setWidth(int valeur) {
        m_size.width = valeur;
    }

    public int getHeight() {
        return m_size.height;
    }

    public void setHeight(int valeur) {
        m_size.height = valeur;
    }

    public Dimension getSize() {
        return m_size;
    }

    public void setSize(Dimension valeur) {
        m_size = valeur;
    }

    public double getRotation() {
        return m_rotation;
    }

    public void setRotation(double valeur) {
        if (Math.abs(valeur) <= 360)
            m_rotation = valeur;
    }

    public String getType() {
        return m_type;
    }

    public void setType(String valeur) {
        m_type = valeur;
    }

    //Fonctions

    public void draw(Graphics g) {

    }

    public boolean hitTest(Point pt) {
        /*
         * AffineTransform tr = new AffineTransform();
         * tr.rotate(this.getRotation()); Polygon pn = new Polygon();
         * pn.addPoint(this.getX(), this.getY()); pn.addPoint(this.getX() +
         * this.getWidth(), this.getY()); pn.addPoint(this.getX() +
         * this.getWidth(), this.getY() + this.getHeight());
         * pn.addPoint(this.getX(), this.getY() + this.getHeight());
         * PathIterator pI = pn.getPathIterator(tr); Polygon p2 =
         * getPolyFromPath(pI); return p2.contains(pt);
         */
        Rectangle rect = new Rectangle(this.getX(), this.getY(), this.getWidth(), this.getHeight());
        return rect.contains(pt);
    }

    public boolean hitTest(Rectangle2D rect) {
        /*
         * AffineTransform tr = new AffineTransform();
         * tr.rotate(this.getRotation()); Polygon pn = new Polygon();
         * pn.addPoint(this.getX(), this.getY()); pn.addPoint(this.getX() +
         * this.getWidth(), this.getY()); pn.addPoint(this.getX() +
         * this.getWidth(), this.getY() + this.getHeight());
         * pn.addPoint(this.getX(), this.getY() + this.getHeight());
         * PathIterator pI = pn.getPathIterator(tr); Polygon p2 =
         * getPolyFromPath(pI); return p2.contains(rect);
         */
        Rectangle rectO = new Rectangle(this.getX(), this.getY(), this.getWidth(), this.getHeight());
        return rect.contains(rectO);
    }

    public Polygon getPolyFromPath(PathIterator pI) {
        Polygon pn = new Polygon();
        while (pI.isDone() == false) {
            double[] coords = new double[6];
            int type = pI.currentSegment(coords);
            switch (type) {
            case PathIterator.SEG_MOVETO:
                pn.addPoint((int) coords[0], (int) coords[1]);
                break;
            case PathIterator.SEG_LINETO:
                pn.addPoint((int) coords[0], (int) coords[1]);
                break;
            }
        }
        return pn;
    }

    public String remplaceVirgule(String a) {
        if (a.indexOf(",") != 0) //$NON-NLS-1$
            a.replace(".", ","); //$NON-NLS-1$ //$NON-NLS-2$
        return a;
    }

}