/*
 * 8 avril 2007
 * 
 * SCALP - Système de Cartographie Automatique Libre pour la Pédagogie
 * Version beta 0.94
 *
 * Copyright 1997-2007 Université de Toulouse Le Mirail / Département de Géographie / Laurent Jégou
 * Portions : 	Copyright (c) 2004 Sun Microsystems (file filter, AbstractLayout)
 * 				Copyright 1999, 2000, 2001, 2002 by Bruno Lowagie (iText pdf exports)
 * 				Copyright (C) 2001 Andrew Khan (jxl import/export format Excelé)
 * http://www.univ-tlse2.fr/geoprdc
 * jegou@univ-tlse2.fr
 * 
 * Ce programme est un logiciel libre régi par la licence CeCILL (cf. lien ci-dessous), cette licence confére
 * aux utilisateurs la liberté de modification et de redistribution du logiciel, dans le cadre d'un modéle de
 * diffusion open-source.
 * L'exercice de ces libertés est assorti de certains devoirs é la charge des utilisateurs afin de préserver
 * ce statut au cours des redistributions ultérieures.
 * Léaccessibilité au code source et les droits de copie, de modification et de redistribution qui en découlent
 * ont pour contrepartie de néoffrir aux utilisateurs quéune garantie limitée et de ne faire peser sur léauteur
 * le titulaire des droits patrimoniaux et les concédants successifs quéune responsabilité restreinte.
 * 
 * Le texte intégral de la licence se trouve dans la disribution, ou é l'adresse suivante :
 * http://www.cecill.info
 */

package guiInterface;

import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;

import javax.swing.JComponent;
import javax.swing.event.MouseInputListener;

import carto.Carte;
import carto.FondCarte;

public class PanneauCarte extends JComponent implements MouseInputListener {

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private FenetreCarte base;

    private FondCarte fondCarte;
    
    private int larg1;

    private int haut1;
    
    private int rx;
    
    private int ry;
    
    private double coeff;
    
    private Carte carte;

    public PanneauCarte(FenetreCarte parent, String action, String fond) {
        addMouseListener(this);
        this.base = parent;
        if (action.equals("importation")) { //$NON-NLS-1$
            this.fondCarte = new FondCarte("importation", fond); // Ouverture //$NON-NLS-1$
            this.carte = this.fondCarte.getCarte();
        } else {
            this.fondCarte = new FondCarte("ouverture", fond); // Ouverture d'un //$NON-NLS-1$
        }
    }

    /*
     * Méthode d'accés au fond de carte
     */
    public FondCarte getFondCarte() {
        return this.fondCarte;
    }

    public void setCarte(Carte carte) {
        this.carte = carte;
    }

    public Carte getCarte() {
        return this.carte;
    }

    /*
     * fonction de dessin du panneau contenant la carte. Dessine le fond de
     * carte redimensionné é la taille de la fenétre
     */
    public void paint(Graphics g) {
        coeff = this.calculCoeff(); //calcul du coeff d'aggrandissement
        if (coeff > 0) {
            fondCarte.calculerCoord(rx, ry, coeff); // redimensionne les coordonnées
            fondCarte.drawFondCarte(g);
        }
        // les coordonnées sont aussi transformées en int
        // et mises dans les tableau xpoints et ypoints de l'objet Polygone
    }

    /*
     * fonction qui calcule le coefficient utilisée pour redimensionner la carte
     * é la taille de la fenétre
     */
    public double calculCoeff() {
        double coeffX;
        double coeffY;
        double toh = 0;
        double tol = 0;
        if (larg1 != 0) {
            tol = larg1;
        } else {
            tol = this.getBounds().getWidth();
        }
        if (haut1 != 0) {
            toh = haut1;
        } else {
            toh = this.getBounds().getHeight();
        }
        larg1 = this.getWidth();
        haut1 = this.getHeight();
        coeffX = ((double) (this.getWidth() - 40)) / fondCarte.getLargeur();
        coeffY = ((double) (this.getHeight() - 40)) / fondCarte.getHauteur();
        if (carte.analyseSymb.cS == 0)
            carte.analyseSymb.cS = 1;
        carte.analyseSymb.cS = carte.analyseSymb.cS * Math.sqrt(Math.abs((this.getHeight() * this.getWidth()) / (toh * tol)));
        if (coeffX > coeffY) {
        	this.rx = 20 + (int) (((this.getWidth() - 40) - (fondCarte.getLargeur() * coeffY)) / 2);
        	this.ry = 20 + (int) (((this.getHeight() - 40) - (fondCarte.getHauteur() * coeffY)) / 2);
            return coeffY;
        }
        else
        {
        	this.rx = 20 + (int) (((this.getWidth() - 40) - (fondCarte.getLargeur() * coeffX)) / 2);
    		this.ry = 20 + (int) (((this.getHeight() - 40)  - (fondCarte.getHauteur() * coeff)) / 2);
            return coeffX;
        }
    }

    /* Fonctions qui définissent les actions souris */
    public void mouseClicked(MouseEvent e) {

    }

    public void mouseEntered(MouseEvent e) {

    }

    public void mouseExited(java.awt.event.MouseEvent mouseEvent) {

    }

    public void mousePressed(java.awt.event.MouseEvent mouseEvent) {
        Point2D click = mouseEvent.getPoint();
        int position = this.fondCarte.appartenir(click);

        //System.out.println("pos: " + position);
        if (position >= 0) {
            //MetaPolygone metaPoly = (MetaPolygone) ((this.fondCarte.getEnsMetaPoly()).get(position));
            //System.out.println(metaPoly.getGeocode() + " " +
            // metaPoly.getNom());
            //int ligne = base.getBase().chercheGeoCode(metaPoly.getGeocode());
        	this.fondCarte.setPSel(position);
        	this.repaint();
            base.getBase().jtableau.changeSelection(position, 0, false, false);
        } else {
        	this.fondCarte.setPSel(-1);
        	this.repaint();
        }
    }

    public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {

    }

    public void mouseDragged(MouseEvent e) {

    }

    public void mouseMoved(MouseEvent e) {

    }

	public double getCoeff() {
		return coeff;
	}
}